<?php
/**
 * 2022 Ship-Quik
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Apache License 2.0
 * that is bundled with this package in the file LICENSE.
 * It is also available through the world-wide-web at this URL:
 * http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * @author    Ship-Quik <info@ship-quik.com>
 * @copyright 2022 Ship-Quik S.L.
 * @license   http://www.apache.org/licenses/LICENSE-2.0.txt  Apache License 2.0
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
define('SHIP_QUIK_PLATFORM', 'prestashop');

define('SHIP_QUIK_NAME', 'ship_quik');
define('SHIP_QUIK_VERSION', '1.0.0');
define('SHIP_QUIK_PLUGIN_PATH', rtrim(_PS_MODULE_DIR_, '/') . '/' . SHIP_QUIK_NAME . '/');
define('SHIP_QUIK_CLASSES_PATH', SHIP_QUIK_PLUGIN_PATH . 'ShipQuik/');

define('CONFIG_DATA', SHIP_QUIK_PLUGIN_PATH . 'config.json');

// Load composer
// #require_once __DIR__ . '/vendor/autoload.php';
require SHIP_QUIK_PLUGIN_PATH . 'vendor/autoload.php';

use ShipQuik\PrestaShop\ShipQuikConfigPage;
use ShipQuik\Shared\ShipQuik_Configuration;
use ShipQuik\Shared\ShipQuik_Helper;
use Tools;

class Ship_Quik extends CarrierModule
{
    /**
     * Id of current carrier. This variable will be set by Presta when
     * method for calculating shipping cost is called.
     *
     * @var int
     */
    public $id_carrier;

    public $_token = '';
    public $config;
    public $_suppliers;

    public $_is_required;

    /**
     * ShipQuik constructor.
     */
    public function __construct()
    {
        // $this->module_key = 'a7a3a395043ca3a09d703f7d1c74a107';
        $this->name = 'ship_quik';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0.0';
        $this->author = 'Ship-Quik S.L.';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->_token = strtoupper('ship_quik');
        $this->config = new ShipQuik_Configuration($this);

        $this->displayName = $this->l('Ship-Quik Shipping');
        $this->description = $this->l('Hacemos fácil tu trabajo comparando todas las opciones de envío, para que puedas contratar las mejores tarifas de transporte en un par de clicks. Reduce tus gastos y controla tus envíos de manera super sencilla.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('SHIP_QUIK_NAME')) {
            $this->warning = $this->l('No name provided');
        }

        $this->_is_required = $this->checkRequiredFields();
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        $sql_file = dirname(__FILE__) . '/install/install.sql';
        if (!$this->loadSQLFile($sql_file)) {
            return false;
        }

        // if (false === (new ShipQuikConfigPage($this->getTranslator()))->run())
        //      return false;

        if (!parent::install()) {
            return false;
        }

        // && $this->registerHook('displayLeftColumn')
        // && $this->registerHook('displayHeader')
        // || !$this->registerHook('actionOrderGridPresenterModifier')

        Configuration::updateValue('SHIP_QUIK_NAME', 'Ship Quik');

        if (!$this->registerHook('actionCarrierUpdate')
            || !$this->registerHook('displayCarrierExtraContent')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->registerHook('actionAdminControllerSetMedia')
            || !$this->registerHook('actionOrderGridDefinitionModifier')
            || !$this->registerHook('actionOrderGridDataModifier')
            || !$this->registerHook('actionOrderGridQueryBuilderModifier')
            || !$this->registerHook('actionAdminOrdersListingFieldsModifier')
            || !$this->registerHook('actionAdminOrdersListingResultsModifier')) {
            return false;
        }

        Tools::clearAllCache();

        return true;
    }

    public function uninstall()
    {
        return parent::uninstall()
            && Configuration::deleteByName('SHIP_QUIK_NAME')
            && $this->uninstallModule();
    }

    public function uninstallModule()
    {
        $sql_file = dirname(__FILE__) . '/install/uninstall.sql';
        if (!$this->loadSQLFile($sql_file)) {
            return false;
        }

        return true;
    }

    public function loadSQLFile($sql_file)
    {
        $sql_content = Tools::file_get_contents($sql_file);

        $sql_content = str_replace('PREFIX_', _DB_PREFIX_, $sql_content);
        $sql_requests = preg_split("/;\s*[\r\n]+/", $sql_content);

        $result = true;
        foreach ($sql_requests as $request) {
            if (!empty($request)) {
                $result &= Db::getInstance()->execute(trim($request));
            }
        }

        return $result;
    }

    public function getHookController($hook_name)
    {
        require_once dirname(__FILE__) . '/controllers/hook/' . $hook_name . '.php';
        $controller_name = $this->name . $hook_name . 'Controller';
        $controller = new $controller_name($this, __FILE__, $this->_path);

        return $controller;
    }

    public function getSupplierServices($name)
    {
        foreach ($this->_suppliers as $supplier) {
            if ($supplier['name'] == $name) {
                return $supplier['services'];
            }
        }

        return [];
    }

    public function replaceStrInArray($search, $replace, $list)
    {
        $result = [];

        foreach ($list as $row) {
            $tmp = [];
            foreach ($row as $key => $element) {
                $tmp[$key] = str_replace($search, $replace, $element);
            }
            $result[] = $tmp;
        }

        return $result;
    }

    public function getContent()
    {
        $output = '';
        $submitName = $this->name;

        if (!Configuration::get('_SHIP_QUIK_activation')) {
            $correct_global = true;

            if (Tools::isSubmit('submit' . $submitName . '_activation')) {
                foreach ($this->getActivationFormFields() as $field => $values) {
                    $correct = true;

                    if ($field == '_SHIP_QUIK_activationEmail') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                            }
                        }
                        if ($correct) {
                            Configuration::updateValue('_SHIP_QUIK_activationEmail', $configValue);
                        } else {
                            break;
                        }
                    }
                    if ($field == '_SHIP_QUIK_activationKey') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                            }
                        }
                        if ($correct) {
                            Configuration::updateValue('_SHIP_QUIK_activationKey', $configValue);
                            $correct_global = true;
                        } else {
                            $correct_global = false;
                            $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                        }
                    }
                }

                if ($correct_global) {
                    $output = $this->displayConfirmation($this->l('Configuración guardada correctamente'));

                    $this->config->reNewAccessToken();
                    $token = unserialize(Configuration::get('_SHIP_QUIK_activation'));

                    if (isset($token->token) && $token->token) {
                        $redirect = Context::getContext()->shop->getBaseURL(true) . end(explode('/', _PS_ADMIN_DIR_)) . '/'
                        . AdminController::$currentIndex
                        . '&' . http_build_query([
                            'configure' => $submitName,
                            'token' => Tools::getValue('token'),
                        ]);
    
                        Tools::redirect($redirect);
                    }
                    $output = $this->displayError($this->l('Ha habido un problema con la activación, revise que la clave privada es correcta.'));
                }
            }

            return $output . $this->displayActivationForm($submitName);
        } else {
            $correct_global = true;

            $this->_suppliers = ShipQuik_Helper::getSuppliersConfiguration($this, []);

            if (Tools::isSubmit('submit' . $submitName)) {
                foreach ($this->getFormFields() as $field => $values) {
                    $correct = true;

                    if ($field == '_SHIP_QUIK_suppliers') {
                        $all_opts = $this->getSuppliersOptions();
                        $checkbox_options = [];
                        foreach ($all_opts as $chbx_options) {
                            $value = Tools::getValue('_SHIP_QUIK_suppliers_' . $chbx_options['id_option']);
                            if ($value) {
                                $checkbox_options[] = [
                                    'name' => $chbx_options['id_option'],
                                    'services' => $this->getSupplierServices($chbx_options['id_option']),
                                ];
                            }
                        }
                        $configValue = serialize($checkbox_options);
                    } elseif ($field == '_SHIP_QUIK_shippingTableRates' || $field == '_SHIP_QUIK_shippingPackageList') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                            }
                        }
                        if ($configValue) {
                            $configValue = $this->replaceStrInArray(',', '.', $configValue);
                        }
                        Configuration::updateValue($field, serialize($configValue));
                        continue;
                    } else {
                        $configValue = (string) Tools::getValue($field);
                    }

                    if ($values['required']) {
                        switch ($values['type']) {
                            case 'serialized':
                                if (empty(unserialize($configValue))) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                                }
                                break;

                            case 'select':
                                if (empty($configValue) && $configValue != '0') {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                                }
                                break;
                            case 'number':
                                if ($configValue <= 0 || !is_numeric($configValue)) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                                } else {
                                    $configValue = str_replace(',', '.', $configValue);
                                }
                                break;
                            case 'text':
                                if (empty($configValue)) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->l('Debe introducir un valor para ' . $field));
                                }
                                break;
                        }
                    }
                    if ($correct) {
                        Configuration::updateValue($field, $configValue);
                    } else {
                        break;
                    }
                }

                if ($correct_global) {
                    $this->installCarriers(Configuration::get('_SHIP_QUIK_pricingMode'));

                    $output = $this->displayConfirmation($this->l('Configuración guardada correctamente'));
                }
            }
        }

        return $output . $this->displayFullForm($submitName);
    }

    public function displayActivationForm($submitName)
    {
        $form1 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('IMPORTANTE'),
                ],
                'input' => [
                    [
                        'type' => 'html',
                        'name' => 'info',
                        'label' => '',
                        'class' => 'panel-activation',
                        'html_content' => '<div class="info-activation">' .
                                '<h4>Una vez tengas instalado el plugin, tienes que ir a nuestra web <a style="color: #00C2A3 !important;" target="_blank" href="https://ship-quik.com/es">Ship-Quik</a> y registrarte para obtener la clave de autorización.</h4>' .
                                '<h4>Verás un apartado para poner el nombre de la tienda y la url de la misma. Una vez estén,  <a style="color: #00C2A3 !important;" target="_blank" href="https://ship-quik.com/es/plugins">haz click en generar Key</a>. Cuando la tengas, cópiala y pégala en el pluging de tu tienda.</h4>' .
                            '</div>',
                        ],
                ],
            ],
        ];

        $form2 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Ship-Quik Licencia y activación'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('Email de registro'),
                        'desc' => $this->l('Introduzca aquí el email con el que se registro en Ship-Quik.'),
                        'name' => '_SHIP_QUIK_activationEmail',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Clave privada'),
                        'desc' => $this->l('Introduzca aquí su clave privada.'),
                        'name' => '_SHIP_QUIK_activationKey',
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Activar'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();

        $helper->table = $this->name;
        $helper->name_controller = $submitName;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $submitName]);
        $helper->submit_action = 'submit' . $submitName . '_activation';

        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

        $helper->fields_value['_SHIP_QUIK_activationEmail'] = '';
        $helper->fields_value['_SHIP_QUIK_activationKey'] = '';

        return $helper->generateForm([$form1, $form2]);
    }

    public function installCarriers($pricingMode)
    {
        $carriers_list = [
            '_SHIP_QUIK_SPEED' => [
                'name' => 'Ship-Quik Envío más rápido',
                'active' => ($pricingMode == 'speed' || $pricingMode == 'speedprice' ? 1 : 0),
            ],
            '_SHIP_QUIK_PRICE' => [
                'name' => 'Ship-Quik Envío más barato',
                'active' => ($pricingMode == 'price' || $pricingMode == 'speedprice' ? 1 : 0),
            ],
        ];

        $id_lang_default = Language::getIsoById(Configuration::get('PS_LANG_DEFAULT'));
        foreach ($carriers_list as $carrier_key => $carrierData) {
            $existCarrierID = Configuration::get($carrier_key);
            if ($existCarrierID) {
                $existCarrier = new Carrier($existCarrierID);
                $existCarrier->name = $carrierData['name'];
                $existCarrier->active = $carrierData['active'];
                foreach (Language::getLanguages(true) as $language) {
                    $existCarrier->delay[(int) $language['id_lang']] = Configuration::get('_SHIP_QUIK_pickUpTimeText');
                }
                $existCarrier->save();
            } else {
                // Create new carrier
                $carrier = new Carrier();
                $carrier->name = $carrierData['name'];
                $carrier->id_tax_rules_group = 0;
                $carrier->active = $carrierData['active'];
                $carrier->deleted = 0;
                foreach (Language::getLanguages(true) as $language) {
                    $carrier->delay[(int) $language['id_lang']] = Configuration::get('_SHIP_QUIK_pickUpTimeText');
                }
                $carrier->shipping_handling = false;
                $carrier->range_behavior = 1;
                $carrier->is_module = true;
                $carrier->shipping_external = true;
                $carrier->external_module_name = $this->name;
                $carrier->shipping_method = 1;
                $carrier->need_range = true;
                if (!$carrier->add()) {
                    return false;
                }

                // Associate carrier to all groups
                $groups = Group::getGroups(true);
                foreach ($groups as $group) {
                    Db::getInstance()->insert('carrier_group', ['id_carrier' => (int) $carrier->id, 'id_group' => (int) $group['id_group']]);
                }

                // Create price range
                $rangePrice = new RangePrice();
                $rangePrice->id_carrier = $carrier->id;
                $rangePrice->delimiter1 = '0';
                $rangePrice->delimiter2 = '999999';
                $rangePrice->add();

                // Create weight range
                $rangeWeight = new RangeWeight();
                $rangeWeight->id_carrier = $carrier->id;
                $rangeWeight->delimiter1 = '0';
                $rangeWeight->delimiter2 = '999999';
                $rangeWeight->add();

                // Associate carrier to all zones
                $zones = Zone::getZones(true);
                foreach ($zones as $zone) {
                    Db::getInstance()->insert('carrier_zone', ['id_carrier' => (int) $carrier->id, 'id_zone' => (int) $zone['id_zone']]);
                    Db::getInstance()->insert('delivery', ['id_carrier' => (int) $carrier->id, 'id_range_price' => (int) $rangePrice->id, 'id_range_weight' => null, 'id_zone' => (int) $zone['id_zone'], 'price' => '0']);
                    Db::getInstance()->insert('delivery', ['id_carrier' => (int) $carrier->id, 'id_range_price' => null, 'id_range_weight' => (int) $rangeWeight->id, 'id_zone' => (int) $zone['id_zone'], 'price' => '0']);
                }

                // Copy the carrier logo
                // copy(dirname(__FILE__).'/views/img/'.$carrier_key.'.jpg', _PS_SHIP_IMG_DIR_.'/'.(int)$carrier->id.'.jpg');

                // Save the carrier ID in the Configuration table
                Configuration::updateValue($carrier_key, $carrier->id);
            }
        }

        return true;
    }

    public function getSuppliersOptions()
    {
        $options = [];

        if ($this->_suppliers) {
            foreach ($this->_suppliers as $supplier) {
                $options[] = [
                    'id_option' => $supplier['name'],
                    'name' => $this->l($supplier['name']),
                ];
            }
        }

        return $options;
    }

    public function getPricingMode()
    {
        $options = [
            [
                'id_option' => 'price',
                'name' => $this->l('Envío más barato'),
            ],
            [
                'id_option' => 'speed',
                'name' => $this->l('Envío más rápido'),
            ],
            [
                'id_option' => 'speedprice',
                'name' => $this->l('El cliente puede elegir entre el más barato o el más rápido'),
            ],
        ];

        return $options;
    }

    public function getPaymentMethod()
    {
        $options = [];

        if ($this->config->getPaymentMethod() == 0) {
            $options[] = [
                'id_option' => 0,
                'name' => $this->l('Fin de mes'),
            ];
        } else {
            $options[] = [
                'id_option' => 1,
                'name' => $this->l('Tarjeta de credito'),
            ];
        }

        return $options;
    }

    public function getFreeShipping()
    {
        $options = [
            [
                'id_option' => 'no',
                'name' => $this->l('No'),
            ],
            [
                'id_option' => 'yes',
                'name' => $this->l('Sí'),
            ],
        ];

        return $options;
    }

    public function getChargeDUA()
    {
        $options = [
            [
                'id_option' => 'yes',
                'name' => $this->l('Sí'),
            ],
            [
                'id_option' => 'no',
                'name' => $this->l('No'),
            ]
        ];

        return $options;
    }

    public function getShopFeeType()
    {
        $options = [
            [
                'id_option' => 'price',
                'name' => $this->l('Por precio'),
            ],
            [
                'id_option' => 'percent',
                'name' => $this->l('Por porcentaje'),
            ],
        ];

        return $options;
    }

    public function getEditableTable($id, $description, $data)
    {
        $data = Configuration::get('_SHIP_QUIK_shippingTableRates');

        if (!$data) {
            $data = [];
        } else {
            $data = unserialize($data);
        }

        $html = '<div id="shippingTableRates" data-option-name="' . $id . '" data-table-count="' . (is_array($data) && !empty($data) ? count($data) : 0) . '" data-delete-text="' . $this->l('¿Estás seguro que quieres borrar rango?.', 'ship-quik') . '">' . "\n";
        $html .= '<div class="btn btn-primary add-rate mt-1">' . $this->l('Añadir Rango', 'ship-quik') . '</div>' . "\n";
        $html .= '<table class="table table-rates">' . "\n";

        $html .= '<tr class="rates-header">';
        $html .= '<td><label><b>' . $this->l('From', 'ship-quik') . '</b></label></th>';
        $html .= '<td><label><b>' . $this->l('Hasta', 'ship-quik') . '</b></label></th>';
        $html .= '<td colspan="2"><label><b>' . $this->l('Importe', 'ship-quik') . '</b></label></th>';
        $html .= '</tr>';

        if (is_array($data)) {
            $i = 0;
            foreach ($data as $element) {
                $html .= '<tr class="rate-row rate-row-' . $i . '">';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][from]" type="text" min="1" step="0.01" value="' . $element['from'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][to]" type="text" min="1" step="0.01" value="' . $element['to'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][value]" type="text" min="1" step="0.01" value="' . $element['value'] . '" /></td>';
                $html .= '<td><span style="cursor: pointer" class="material-icons delete-rate" data-id="' . $i . '" >delete</span></td>';
                $html .= '</tr>';

                ++$i;
            }
        }
        $html .= '</table>' . "\n";

        $html .= '<span class="description">' . $description . '</span>' . "\n";

        $html .= '</div>' . "\n";

        return $html;
    }

    public function getProductListWithoutParcel($description)
    {
        $html = '';
        $limit = 15;
        $more = false;

        $products = Product::getProducts(Context::getContext()->language->getId(), 1, -1, 'id_product', 'ASC');

        $productsNeedParcelData = [];

        foreach ($products as $element) {
            $product = new Product($element['id_product']);

            $name = Product::getProductName($element['id_product']);
            $link = Context::getContext()->link->getAdminLink('AdminProducts', true, ['id_product' => $element['id_product']]);

            $width = (float) $element['width'];
            $height = (float) $element['height'];
            $depth = (float) $element['depth'];
            $weight = (float) $element['weight'];

            if ($weight == 0 || $depth == 0 || $width == 0 || $height == 0) {
                $productsNeedParcelData[] = [
                    'name' => $name,
                    'width' => $width,
                    'height' => $height,
                    'depth' => $depth,
                    'weight' => $weight,
                    'link' => $link,
                ];
            }
        }

        if (count($productsNeedParcelData) > 0) {
            $html .= '<div id="shippingProductListWhithoutParcelData">' . "\n";
            $html .= '<h2><b>' . $this->l('Atención: los datos de embalaje de estos productos no están completamente informados.', 'ship-quik') . '</b></h2>';

            $html .= '<table class="table">' . "\n";
            $html .= '<tr class="rates-header">';
            $html .= '<td><label><b>' . $this->l('Nombre del producto', 'ship-quik') . '</b></label></th>';
            $html .= '<td style="text-align: center;"><label><b>' . $this->l('Anchura', 'ship-quik') . '</b></label></th>';
            $html .= '<td style="text-align: center;"><label><b>' . $this->l('Altura', 'ship-quik') . '</b></label></th>';
            $html .= '<td style="text-align: center;"><label><b>' . $this->l('Profundidad', 'ship-quik') . '</b></label></th>';
            $html .= '<td style="text-align: center;"><label><b>' . $this->l('Peso', 'ship-quik') . '</b></label></th>';
            $html .= '<td style="text-align: center;"><label><b>' . $this->l('Página del producto', 'ship-quik') . '</b></label></th>';
            $html .= '</tr>';

            foreach ($productsNeedParcelData as $i => $element) {
                if ($i == $limit) {
                    $more = true;
                    break;
                }
                $html .= '<tr class="rates-header">';
                $html .= '<td><b><a href="' . $element['link'] . '#tab-step4">' . $element['name'] . '</a></b></td>';
                $html .= '<td style="text-align: center;">' . $element['width'] . '</td>';
                $html .= '<td style="text-align: center;">' . $element['height'] . '</td>';
                $html .= '<td style="text-align: center;">' . $element['depth'] . '</td>';
                $html .= '<td style="text-align: center;">' . $element['weight'] . '</td>';

                $html .= '<td style="text-align: center;"><b><a href="' . $element['link'] . '#tab-step4">' . $this->l('Ver', 'ship-quik') . '</a></b></td>';
                $html .= '</tr>';
            }

            $html .= '</table>' . "\n";

            $more ? $html .= '<span class="description">' . $this->l('Tienes más productos sin datos, solo hemos mostrado los ' . $limit . ' primeros.', 'ship-quik') . '</span>' . "\n" : '';

            $html .= '<span class="description">' . $description . '</span>' . "\n";

            $html .= '</div>' . "\n";
        }

        return $html;
    }

    public function getPackageList($id, $description, $data)
    {
        $data = Configuration::get('_SHIP_QUIK_shippingPackageList');

        if (!$data) {
            $data = [];
        } else {
            $data = unserialize($data);
        }

        $html = '<div id="shippingPackageList" data-option-name="' . $id . '" data-table-count="' . (is_array($data) && !empty($data) ? count($data) : 0) . '" data-delete-text="' . $this->l('¿Estás seguro que quieres borrar este paquete?.', 'ship-quik') . '">' . "\n";
        $html .= '<div class="btn btn-primary add-rate mt-1">' . $this->l('Añadir Paquete', 'ship-quik') . '</div>' . "\n";
        $html .= '<table class="table table-rates">' . "\n";

        $html .= '<tr class="rates-header">';
        $html .= '<td><label><b>' . $this->l('Nombre', 'ship-quik') . '</b></label></th>';
        $html .= '<td><label><b>' . $this->l('Largo (cm)', 'ship-quik') . '</b></label></th>';
        $html .= '<td><label><b>' . $this->l('Ancho (cm)', 'ship-quik') . '</b></label></th>';
        $html .= '<td><label><b>' . $this->l('Alto (cm)', 'ship-quik') . '</b></label></th>';
        $html .= '<td><label><b>' . $this->l('Peso (kgr)', 'ship-quik') . '</b></label></th>';
        $html .= '<td colspan="2"><label><b>' . $this->l('Paquete por defecto', 'ship-quik') . '</b></label></th>';
        $html .= '</tr>';

        if (is_array($data)) {
            $i = 0;
            foreach ($data as $element) {
                $isChecked = false;

                if (isset($element['default'])) {
                    if ($element['default'] == 'on') {
                        $isChecked = true;
                    }
                }

                $html .= '<tr class="rate-row rate-row-' . $i . '">';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][name]" type="text" value="' . $element['name'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][depth]" type="text" min="1" step="0.01" value="' . $element['depth'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][width]" type="text" min="1" step="0.01" value="' . $element['width'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][height]" type="text" min="1" step="0.01" value="' . $element['height'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][weight]" type="text" min="1" step="0.01" value="' . $element['weight'] . '" /></td>';
                $html .= '<td><input class="field" name="' . $id . '[' . $i . '][default]" type="checkbox" ' . ($isChecked ? 'checked=checked' : '') . '" /></td>';
                $html .= '<td><span style="cursor: pointer" class="material-icons delete-rate" data-id="' . $i . '" >delete</span></td>';
                $html .= '</tr>';

                ++$i;
            }
        }
        $html .= '</table>' . "\n";

        $html .= '<span class="description">' . $description . '</span>' . "\n";

        $html .= '</div>' . "\n";

        return $html;
    }

    public function displayFullForm($submitName)
    {
        $logo = Tools::file_get_contents(SHIP_QUIK_PLUGIN_PATH . 'views/img/logos/logo.base64');

        $form1 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Lista de proveedores'),
                ],
                'input' => [
                    [
                        'type' => 'checkbox',
                        'desc' => $this->l('Escoje con qué proveedores quieres trabajar'),
                        'name' => '_SHIP_QUIK_suppliers',
                        'required' => true,
                        'values' => [
                            'query' => $this->getSuppliersOptions(),
                            'id' => 'id_option',
                            'name' => 'name',
                            'expand' => [
                                'print_total' => count($this->getSuppliersOptions()),
                                'default' => 'show',
                                'show' => ['text' => $this->l('show'), 'icon' => 'plus-sign-alt'],
                                'hide' => ['text' => $this->l('hide'), 'icon' => 'minus-sign-alt'],
                            ],
                        ],
                    ],
                ],
            ],
        ];

        $form2 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Método de pago'),
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'name' => '_SHIP_QUIK_paymentMethod',
                        'desc' => 'Esta opción le permitirá pagar sus pedidos a fin de mes. Al hacer clic, recibiremos un correo electrónico con su solicitud. Tendrás que firmar el documento SEPA y cuando esté autorizado, podrás pagar a final de mes. Si quieres más información envíanos un email a <a href="mailto:vamos@ship-quik.com">vamos@ship-quik.com</a>',
                        'required' => true,
                        'options' => [
                            'query' => $this->getPaymentMethod(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                ],
            ],
        ];

        $form3 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Tarifas'),
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'label' => $this->l('Selección de precio'),
                        'name' => '_SHIP_QUIK_pricingMode',
                        'required' => true,
                        'class' => 'subsection',
                        'options' => [
                            'query' => $this->getPricingMode(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Cargar costes de importación a tus clientes'),
                        'name' => '_SHIP_QUIK_chargeDUA',
                        'desc' => $this->l('¿Quieres cargar costes de importación a tus clientes?, si es Sí, este coste se cargará incluso en los envíos gratuitos.'),
                        'required' => true,
                        'options' => [
                            'query' => $this->getChargeDUA(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Texto en Tiempo de tránsito'),
                        'name' => '_SHIP_QUIK_pickUpTimeText',
                        'required' => true,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Envío gratuito'),
                        'name' => '_SHIP_QUIK_isFreeShipping',
                        'desc' => $this->l('Si quieres ofrecer envío gratuito a tus clientes'),
                        'required' => false,
                        'options' => [
                            'query' => $this->getFreeShipping(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Envío gratuito desde'),
                        'name' => '_SHIP_QUIK_freeShippingPriceFrom',
                        'desc' => $this->l('Subtotal de carrito igual o superior a este importe'),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Tarifas por rango de importes'),
                        'name' => '_SHIP_QUIK_shippingTableRates',
                        'desc' => 'Define tu tarifa dependiendo del subtotal del carrito',
                        'html_content' => $this->getEditableTable('_SHIP_QUIK_shippingTableRates', '', Configuration::get('_SHIP_QUIK_shippingTableRates')),
                        'required' => false,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Tipo de comisión de la tienda'),
                        'name' => '_SHIP_QUIK_shopFeeType',
                        'required' => false,
                        'options' => [
                            'query' => $this->getShopFeeType(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Valor de la comisión'),
                        'name' => '_SHIP_QUIK_shopFeeValue',
                        'required' => false,
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->l('Comentarios de la tienda sobre el envío'),
                        'name' => '_SHIP_QUIK_comments',
                        'required' => false,
                    ],
                ],
            ],
        ];

        $form4 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Dirección del remitente'),
                ],
                'input' => [
                    [
                        'type' => 'html',
                        'name' => 'sander_section',
                        'html_content' => '<h1>' . $this->l('Dirección del remitente') . '</h1>',
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Empresa'),
                        'name' => '_SHIP_QUIK_senderAddressCompany',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Dirección'),
                        'name' => '_SHIP_QUIK_senderAddressStreet',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Ciudad'),
                        'name' => '_SHIP_QUIK_senderAddressCity',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Código Postal'),
                        'name' => '_SHIP_QUIK_senderAddressPostalCode',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Código de país'),
                        'name' => '_SHIP_QUIK_senderAddressCountryCode',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Número de teléfono'),
                        'name' => '_SHIP_QUIK_senderAddressPhoneNumber',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Email'),
                        'name' => '_SHIP_QUIK_senderAddressEmail',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('CIF/DNI/NIE'),
                        'name' => '_SHIP_QUIK_senderAddressCif',
                        'required' => true,
                    ],
                ],
            ],
        ];

        $form5 = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Paquetes predeterminados'),
                ],
                'input' => [
                    [
                        'type' => 'html',
                        'name' => '_SHIP_QUIK_shippingProductListWithoutParcelData',
                        'html_content' => $this->getProductListWithoutParcel(''),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Lista de paquetes'),
                        'name' => '_SHIP_QUIK_shippingPackageList',
                        'desc' => 'Define los paquetes de que dispones para tus envíos',
                        'html_content' => $this->getPackageList('_SHIP_QUIK_shippingPackageList', '', Configuration::get('_SHIP_QUIK_shippingPackageList')),
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();

        $helper->table = $this->name;
        $helper->name_controller = $submitName;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $submitName]);
        $helper->submit_action = 'submit' . $submitName;

        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

        if (Configuration::get('_SHIP_QUIK_suppliers')) {
            $suppliers = unserialize(Configuration::get('_SHIP_QUIK_suppliers'));
            foreach ($suppliers as $supplier) {
                $helper->fields_value['_SHIP_QUIK_suppliers_' . $supplier['name']] = true;
            }
        } else {
            $helper->fields_value['_SHIP_QUIK_suppliers'] = Tools::getValue('_SHIP_QUIK_suppliers', Configuration::get('_SHIP_QUIK_suppliers'));
        }

        $helper->fields_value['_SHIP_QUIK_pricingMode'] = Tools::getValue('_SHIP_QUIK_pricingMode', Configuration::get('_SHIP_QUIK_pricingMode'));
        $helper->fields_value['_SHIP_QUIK_chargeDUA'] = Tools::getValue('_SHIP_QUIK_chargeDUA', Configuration::get('_SHIP_QUIK_chargeDUA'));
        $helper->fields_value['_SHIP_QUIK_paymentMethod'] = Tools::getValue
        ('_SHIP_QUIK_paymentMethod', Configuration::get('_SHIP_QUIK_paymentMethod'));
        $helper->fields_value['_SHIP_QUIK_isFreeShipping'] = Tools::getValue('_SHIP_QUIK_isFreeShipping', Configuration::get('_SHIP_QUIK_isFreeShipping'));
        $helper->fields_value['_SHIP_QUIK_freeShippingPriceFrom'] = Tools::getValue('_SHIP_QUIK_freeShippingPriceFrom', Configuration::get('_SHIP_QUIK_freeShippingPriceFrom') ? Configuration::get('_SHIP_QUIK_freeShippingPriceFrom') : '0');
        $helper->fields_value['_SHIP_QUIK_shopFeeType'] = Tools::getValue('_SHIP_QUIK_shopFeeType', Configuration::get('_SHIP_QUIK_shopFeeType'));
        $helper->fields_value['_SHIP_QUIK_shopFeeValue'] = Tools::getValue('_SHIP_QUIK_shopFeeValue', Configuration::get('_SHIP_QUIK_shopFeeValue') ? Configuration::get('_SHIP_QUIK_shopFeeValue') : 0);
        $helper->fields_value['_SHIP_QUIK_comments'] = Tools::getValue('_SHIP_QUIK_comments', Configuration::get('_SHIP_QUIK_comments') ? Configuration::get('_SHIP_QUIK_comments') : '');
        $helper->fields_value['_SHIP_QUIK_pickUpTimeText'] = Tools::getValue('_SHIP_QUIK_pickUpTimeText', Configuration::get('_SHIP_QUIK_pickUpTimeText') ? Configuration::get('_SHIP_QUIK_pickUpTimeText') : '');

        $helper->fields_value['_SHIP_QUIK_senderAddressCompany'] = Tools::getValue('_SHIP_QUIK_senderAddressCompany', Configuration::get('_SHIP_QUIK_senderAddressCompany'));
        $helper->fields_value['_SHIP_QUIK_senderAddressStreet'] = Tools::getValue('_SHIP_QUIK_senderAddressStreet', Configuration::get('_SHIP_QUIK_senderAddressStreet'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCity'] = Tools::getValue('_SHIP_QUIK_senderAddressCity', Configuration::get('_SHIP_QUIK_senderAddressCity'));
        $helper->fields_value['_SHIP_QUIK_senderAddressPostalCode'] = Tools::getValue('_SHIP_QUIK_senderAddressPostalCode', Configuration::get('_SHIP_QUIK_senderAddressPostalCode'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCountryCode'] = Tools::getValue('_SHIP_QUIK_senderAddressCountryCode', Configuration::get('_SHIP_QUIK_senderAddressCountryCode') ? Configuration::get('_SHIP_QUIK_senderAddressCountryCode') : 'ES');
        $helper->fields_value['_SHIP_QUIK_senderAddressPhoneNumber'] = Tools::getValue('_SHIP_QUIK_senderAddressPhoneNumber', Configuration::get('_SHIP_QUIK_senderAddressPhoneNumber'));
        $helper->fields_value['_SHIP_QUIK_senderAddressEmail'] = Tools::getValue('_SHIP_QUIK_senderAddressEmail', Configuration::get('_SHIP_QUIK_senderAddressEmail'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCif'] = Tools::getValue('_SHIP_QUIK_senderAddressCif', Configuration::get('_SHIP_QUIK_senderAddressCif'));

        return $helper->generateForm([$form1, $form2, $form3, $form4, $form5]);
    }

    private function checkRequiredFields()
    {
        $required_fields = [
            'suppliers',
            'pricingMode',
            'chargeDUA',
            'pickUpTimeText',
            'senderAddressCompany',
            'senderAddressStreet',
            'senderAddressCity',
            'senderAddressPostalCode',
            'senderAddressCountryCode',
            'senderAddressPhoneNumber',
            'senderAddressEmail',
            'senderAddressCif',
            'shippingPackageList',
        ];

        foreach ($required_fields as $field) {
            $value = $this->config->getValueByKey($field);
            if (!$value) {
                return true;
            }
        }

        return false;
    }

    /* Carrier & Shipping */

    public function hookActionCarrierUpdate($params)
    {
        $controller = $this->getHookController('actionCarrierUpdate');

        return $controller->run($params);
    }

    public function hookDisplayCarrierExtraContent($params)
    {
        $controller = $this->getHookController('displayCarrierExtraContent');

        return $controller->run($params);
    }

    public function getOrderShippingCost($params, $shipping_cost)
    {
        $controller = $this->getHookController('getOrderShippingCost');

        return $controller->run($params, $shipping_cost);
    }

    public function getOrderShippingCostExternal($params)
    {
        return $this->getOrderShippingCost($params, 0);
    }

    /* Orders */

    public function hookDisplayOrderConfirmation($params)
    {
        $controller = $this->getHookController('displayOrderConfirmation');

        return $controller->run($params);
    }

    public function hookDisplaydisplayAdminGridTableAfter($params)
    {
        $controller = $this->getHookController('displayAdminGridTableAfter');

        return $controller->run($params);
    }

    public function hookActionOrderGridDefinitionModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridDefinitionModifier');

        return $controller->run($params);
    }

    public function hookActionOrderGridDataModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridDataModifier');

        return $controller->run($params);
    }

    public function hookActionOrderGridQueryBuilderModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridQueryBuilderModifier');

        return $controller->run($params);
    }

    public function hookActionAdminOrdersListingFieldsModifier($params)
    {
        $controller = $this->getHookController('actionAdminOrdersListingFieldsModifier');

        return $controller->run($params);
    }

    public function hookActionAdminOrdersListingResultsModifier($params)
    {
        $controller = $this->getHookController('actionAdminOrdersListingResultsModifier');

        return $controller->run($params);
    }

    // TODO: No se usa, es para versiones menores que la 1.7.7.0
    public function hookActionOrderGridPresenterModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridPresenterModifier');

        return $controller->run($params);
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        $controller = $this->getHookController('actionAdminControllerSetMedia');

        return $controller->run($params);
    }

    public function getActivationFormFields()
    {
        return [
            '_SHIP_QUIK_activationKey' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_activationEmail' => ['required' => true, 'type' => 'text'],
        ];
    }

    public function getFormFields()
    {
        return [
            '_SHIP_QUIK_suppliers' => ['required' => true, 'type' => 'serialized'],
            '_SHIP_QUIK_pricingMode' => ['required' => true, 'type' => 'select'],
            '_SHIP_QUIK_chargeDUA' => ['required' => true, 'type' => 'select'],
            '_SHIP_QUIK_paymentMethod' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_isFreeShipping' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_shippingTableRates' => ['required' => false, 'type' => 'html'],
            '_SHIP_QUIK_freeShippingPriceFrom' => ['required' => false, 'type' => 'number'],
            '_SHIP_QUIK_shopFeeType' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_shopFeeValue' => ['required' => false, 'type' => 'number'],
            '_SHIP_QUIK_comments' => ['required' => false, 'type' => 'text'],
            '_SHIP_QUIK_pickUpTimeText' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCompany' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressStreet' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCity' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressPostalCode' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCountryCode' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressPhoneNumber' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressEmail' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCif' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_shippingPackageList' => ['required' => true, 'type' => 'html'],
        ];
    }
}
