<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\RequestException;

class ShipQuik_API
{
    private static $_instance = null;

    protected $_params = [];

    protected $parent = null;

    protected $config = null;

    protected $_is_post = true;

    public function __construct($parent = null, $params = [], $is_post = true)
    {
        $this->parent = $parent;
        $this->_params = $params;
        $this->_is_post = $is_post;

        $this->config = $parent->config;
    }

    public function runCall($connecttimeout = 800, $timeout = 800)
    {
        try {
            $client = new Client();

            $headers = [
                'Content-Type' => 'application/json; charset=utf-8',
            ];

            $token = $this->config->getAccessToken();

            if ($token) {
                $headers['access-token'] = $token;
            }

            try {
                if ($this->_is_post) {
                    $request = $client->createRequest('POST', $this->_params['Baseurl'], [
                        'headers' => $headers,
                        'body' => json_encode($this->_params['Data']),
                    ]);
                } else {
                    $request = $client->createRequest('GET', $this->_params['Baseurl'], [
                        'headers' => $headers,
                        'query' => $this->_params['Data']
                    ]);
                }

                $response = $client->send($request);

            } catch (RequestException $e) {
                if ($e->getCode() == 401) {
                    return 'renew_token';
                }
                $response = $e->getResponse();
                if ($response) {
                    $message = json_decode($response->getBody()->getContents(), true);
                    if ($message) {
                        \PrestaShopLogger::addLog($message['code'] . ':' . $message['message'], 2);
                    }
                } else {
                    $message = $e->getMessage();
                    if ($message) {
                        \PrestaShopLogger::addLog('Ship-Quik:' . $message, 4);
                    }
                }

                return false;
            }

            $data = json_decode($response->getBody()->getContents(), true);

            return $data;
        } catch (ConnectException $e) {
            if ($message) {
                \PrestaShopLogger::addLog($message['code'] . ':' . $message['message'], 2);
            }
        }

        unset($response);
        unset($client);

        return false;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
