<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

use ShipQuik\PrestaShop\ShipQuik_Shipments;

class ShipQuik_ShippingPickupInfo
{
    private static $_instance = null;

    protected $parent;

    protected $pickupInfo;

    protected $config;

    protected $data;

    public function __construct($parent = null, $pickupInfo = [])
    {
        $this->parent = $parent;
        $this->pickupInfo = $pickupInfo;

        $this->config = $parent->config;

        $this->data = $this->getData($this->pickupInfo);
    }

    private function getData($pickupInfo)
    {
        $pickupInfoData = [];

        $customerAddress = $pickupInfo['destination'];

        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $pickupInfoData['supplier'] = $pickupInfo['supplier'];
        $pickupInfoData['service'] = $pickupInfo['service'];

        $pickupInfoData['origin']['countryCode'] = $senderAddress['CountryCode'];
        $pickupInfoData['origin']['postalCode'] = $senderAddress['PostalCode'];
        $pickupInfoData['destination']['countryCode'] = $customerAddress['country'];
        $pickupInfoData['destination']['postalCode'] = $customerAddress['postcode'];

        $pickupInfoData['isKG'] = true;

        $pickupInfoData['parcels'] = $pickupInfo['boxes'];

        return $pickupInfoData;
    }

    public function send()
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['getSuppliersPickupInfo'];
        $params['Data'] = $this->data;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            if (isset($result['data'])) {
                return $result;
            }
        }

        return $result;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
