<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

use PrestaShopBundle\Controller\Admin\FrameworkBundleAdminController;
use ShipQuik\Shared\ShipQuik_Helper;

class ShipQuikBulkController extends FrameworkBundleAdminController
{
    public function bulkOrderShipment()
    {
        $correct = [];
        $incorrect = [];
        $selectedOrder = \Tools::getValue('order_orders_bulk');
        if (is_array($selectedOrder)) {
            foreach ($selectedOrder as $order_id) {
                $order = new \Order((int) $order_id);
                $result = ShipQuik_BD::getDatabyOrder($order);

                if ($result) {
                    if (!empty($result['pickup_datetime']) && empty($result['shipment_data'])) {
                        $shiQuikShipments = new ShipQuik_Shipments(\Module::getInstanceByName('ship_quik'), $order_id);
                        if (!empty($shiQuikShipments->data)) {
                            $result = $shiQuikShipments->send();
                            if ($result && $result['status'] == 200) {
                                $order = new \Order((int) $order_id);

                                $sql = ShipQuik_BD::setShipmentDatabyOrder($order, $result);

                                $correct[] = $order_id;
                            } else {
                                $incorrect[] = $order_id;
                            }
                        }
                    }
                }
            }
        }
        if (!empty($correct)) {
            $this->addFlash('success', $this->trans('Ship-Quik Pedidos enviados correctamente: ', 'Admin.Notifications.Success') . implode(',', $correct));
        }
        if (!empty($incorrect)) {
            $this->addFlash('error', $this->trans('Ship-Quik Pedidos con problemas : ', 'Admin.Notifications.Error') . implode(',', $incorrect));
        }

        return $this->redirectToRoute('admin_orders_index');
    }

    public function bulkStripePaymentSelectedOrders()
    {
        $parent = \Module::getInstanceByName('ship_quik');
        $selectedOrders = \Tools::getValue('order_orders_bulk');

        $totalPayment = 0;
        $paymentOredersId = [];

        foreach ($selectedOrders as $order_id) {
            $order = new \Order((int) $order_id);
            $result = ShipQuik_BD::getDatabyOrder($order);

            if ($result) {
                if (!empty($result['pickup_datetime']) && empty($result['shipment_data'])) {
                    $orderData = ShipQuik_BD::getDatabyOrderId($order_id);
                    $order = new \Order($order_id);
                    $shipment = json_decode($orderData['total_payment'], true);

                    if ($orderData && $order) {
                        if ($orderData['paid'] == '0' && (float) $shipment['payment']['total'] > 0) {
                            $totalPayment = $totalPayment + (float) $shipment['payment']['total'];
                        }
                    }
                }
            }
        }
        if ($totalPayment > 0) {
            $senderAddress = ShipQuik_Shipments::getSenderAddress($parent->config);

            $data = [
                'currency' => 'eur',
                'amount' => (int) ($totalPayment * 100),
                'payment_method_types' => ['card'],
                'metadata' => [
                    'Nro de Orden', json_encode($selectedOrders),
                ],
                'setup_future_usage' => 'off_session',
                'customer' => [
                    'email' => $parent->config->get_option('PS_SHOP_EMAIL'),
                    'name' => $parent->config->getValueByKey('senderAddressCompany'),
                    'phone' => $parent->config->getValueByKey('senderAddressPhoneNumber'),
                    'metadata' => [
                        'cif' => $parent->config->getValueByKey('senderAddressCif'),
                        'address' => $senderAddress['Street'],
                    ],
                ],
            ];

            $stripeIntent = ShipQuik_Helper::createStripeIntent($parent, $data);
            $returnUrl = \Context::getContext()->shop->getBaseURL(true) . ltrim($this->redirectToRoute('admin_orders_index')->getTargetUrl(), '/');

            if ($stripeIntent) {
                $requestParams = [
                    'ordersId' => implode(',', $selectedOrders),
                    'paymentTotal' => $totalPayment,
                    'paymentEmail' => $parent->config->get_option('PS_SHOP_EMAIL'),
                    'paymentKey' => (isset($stripeIntent['clientSecret']) ? $stripeIntent['clientSecret'] : 'nopaymentkey'),
                    'paymentReturnCancelUrl' => $returnUrl,
                    'paymentReturnSubmitUrl' => $returnUrl,
                ];

                return $this->redirectToRoute('admin_orders_index', $requestParams);
            }
        }

        return $this->redirectToRoute('admin_orders_index');
    }
}
