<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class ShipQuik_MultiCurl
{
    private static $_instance = null;

    protected $parent;

    protected $config = null;

    protected $_params = [];

    protected $_is_post = true;

    protected $_verbose = false;

    protected $suppliers = [];

    public function __construct($parent = null, $params = [], $verbose = false, $is_post = true)
    {
        $this->parent = $parent;
        $this->_params = $params;
        $this->_verbose = $verbose;
        $this->_is_post = $is_post;

        $this->config = $parent->config;
    }

    public function getCurtHandle($supplier, $connecttimeout = 800, $timeout = 800, $params = [])
    {
        $curl = curl_init($supplier);
        curl_setopt($curl, CURLOPT_URL, $params['Baseurl']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_NOSIGNAL, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $connecttimeout);
        curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);

        $opciones = [
            'Content-Type: application/json; charset=utf-8',
            'Content-Length: ' . strlen(json_encode($params['Data'])),
            'Expect: ',
        ];

        if ($this->_is_post) {
            curl_setopt($curl, CURLOPT_POST, $this->_is_post);
            curl_setopt(
                $curl,
                CURLOPT_POSTFIELDS,
                json_encode($params['Data'])
            );
            curl_setopt(
                $curl,
                CURLOPT_HTTPHEADER,
                $opciones
            );
        }

        if ($this->_verbose) {
            curl_setopt($curl, CURLOPT_VERBOSE, true);
        }

        return $curl;
    }

    public function curlCall($connecttimeout = 800, $timeout = 800)
    {
        $multiCurl = curl_multi_init();
        $handleList = [];

        foreach ($this->_params as $supplier => $param) {
            $handleList[$supplier] = $this->getCurtHandle($supplier, $connecttimeout, $timeout, $param);
            curl_multi_add_handle($multiCurl, $handleList[$supplier]);
        }

        $running = null;
        do {
            curl_multi_exec($multiCurl, $running);
        } while ($running);

        foreach ($this->_params as $supplier => $param) {
            curl_multi_remove_handle($multiCurl, $handleList[$supplier]);
        }

        curl_multi_close($multiCurl);

        $result = [];

        foreach ($this->_params as $supplier => $param) {
            $result[$supplier] = json_decode(curl_multi_getcontent($handleList[$supplier]), true);
        }

        $resultArray = [];
        foreach ($result as $supplier => $content) {
            if (isset($content['data'])) {
                foreach ($content['data'] as $service) {
                    $resultArray[] = $service;
                }
            }
        }

        return $resultArray;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
